package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.PgDB;

/**
 * PostgreSQL integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("Core | Java | PostgreSQL: Json")
final public class PostgreSQLJsonIT {

    @Test
    @DisplayName("all retrieves all documents")
    public void allDefault() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.allDefault(db);
        }
    }

    @Test
    @DisplayName("all succeeds with an empty table")
    public void allEmpty() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.allEmpty(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    public void byIdString() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byIdString(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    public void byIdNumber() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byIdNumber(db);
        }
    }

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    public void byIdNotFound() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byIdNotFound(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents")
    public void byFieldsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    public void byFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    public void byFieldsMatchNumIn() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatchNumIn(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    public void byFieldsMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    public void byFieldsNoMatchInArray() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byFieldsNoMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byContains retrieves matching documents")
    public void byContainsMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byContainsMatch(db);
        }
    }

    @Test
    @DisplayName("byContains retrieves ordered matching documents")
    public void byContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byContains succeeds when no documents match")
    public void byContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byJsonPath retrieves matching documents")
    public void byJsonPathMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byJsonPathMatch(db);
        }
    }

    @Test
    @DisplayName("byJsonPath retrieves ordered matching documents")
    public void byJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byJsonPath succeeds when no documents match")
    public void byJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.byJsonPathNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    public void firstByFieldsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    public void firstByFieldsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    public void firstByFieldsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    public void firstByFieldsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document")
    public void firstByContainsMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document among many")
    public void firstByContainsMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByContains retrieves a matching document among many (ordered)")
    public void firstByContainsMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByContains returns null when no document matches")
    public void firstByContainsNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByContainsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document")
    public void firstByJsonPathMatchOne() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many")
    public void firstByJsonPathMatchMany() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
    public void firstByJsonPathMatchOrdered() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByJsonPath returns null when no document matches")
    public void firstByJsonPathNoMatch() throws DocumentException {
        try (PgDB db = new PgDB()) {
            JsonFunctions.firstByJsonPathNoMatch(db);
        }
    }
}
