package solutions.bitbadger.documents.core.tests.java.integration;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import solutions.bitbadger.documents.DocumentException;
import solutions.bitbadger.documents.core.tests.integration.SQLiteDB;

import static org.junit.jupiter.api.Assertions.assertThrows;

/**
 * SQLite integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("Core | Java | SQLite: Json")
final public class SQLiteJsonIT {

    @Test
    @DisplayName("all retrieves all documents")
    public void allDefault() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.allDefault(db);
        }
    }

    @Test
    @DisplayName("all succeeds with an empty table")
    public void allEmpty() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.allEmpty(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    public void byIdString() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byIdString(db);
        }
    }

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    public void byIdNumber() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byIdNumber(db);
        }
    }

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    public void byIdNotFound() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byIdNotFound(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents")
    public void byFieldsMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byFieldsMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    public void byFieldsMatchOrdered() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    public void byFieldsMatchNumIn() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byFieldsMatchNumIn(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match")
    public void byFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    public void byFieldsMatchInArray() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byFieldsMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    public void byFieldsNoMatchInArray() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.byFieldsNoMatchInArray(db);
        }
    }

    @Test
    @DisplayName("byContains fails")
    public void byContainsFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> JsonFunctions.byContainsMatch(db));
        }
    }

    @Test
    @DisplayName("byJsonPath fails")
    public void byJsonPathFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> JsonFunctions.byJsonPathMatch(db));
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    public void firstByFieldsMatchOne() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.firstByFieldsMatchOne(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    public void firstByFieldsMatchMany() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.firstByFieldsMatchMany(db);
        }
    }

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    public void firstByFieldsMatchOrdered() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.firstByFieldsMatchOrdered(db);
        }
    }

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    public void firstByFieldsNoMatch() throws DocumentException {
        try (SQLiteDB db = new SQLiteDB()) {
            JsonFunctions.firstByFieldsNoMatch(db);
        }
    }

    @Test
    @DisplayName("firstByContains fails")
    public void firstByContainsFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> JsonFunctions.firstByContainsMatchOne(db));
        }
    }

    @Test
    @DisplayName("firstByJsonPath fails")
    public void firstByJsonPathFails() {
        try (SQLiteDB db = new SQLiteDB()) {
            assertThrows(DocumentException.class, () -> JsonFunctions.firstByJsonPathMatchOne(db));
        }
    }
}
