package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test

/**
 * PostgreSQL integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName('Groovy | PostgreSQL: Json')
final class PostgreSQLJsonIT {

    @Test
    @DisplayName('all retrieves all documents')
    void allDefault() {
        new PgDB().withCloseable JsonFunctions.&allDefault
    }

    @Test
    @DisplayName('all succeeds with an empty table')
    void allEmpty() {
        new PgDB().withCloseable JsonFunctions.&allEmpty
    }

    @Test
    @DisplayName('byId retrieves a document via a string ID')
    void byIdString() {
        new PgDB().withCloseable JsonFunctions.&byIdString
    }

    @Test
    @DisplayName('byId retrieves a document via a numeric ID')
    void byIdNumber() {
        new PgDB().withCloseable JsonFunctions.&byIdNumber
    }

    @Test
    @DisplayName('byId returns null when a matching ID is not found')
    void byIdNotFound() {
        new PgDB().withCloseable JsonFunctions.&byIdNotFound
    }

    @Test
    @DisplayName('byFields retrieves matching documents')
    void byFieldsMatch() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields retrieves ordered matching documents')
    void byFieldsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatchOrdered
    }

    @Test
    @DisplayName('byFields retrieves matching documents with a numeric IN clause')
    void byFieldsMatchNumIn() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatchNumIn
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byFields retrieves matching documents with an IN_ARRAY comparison')
    void byFieldsMatchInArray() {
        new PgDB().withCloseable JsonFunctions.&byFieldsMatchInArray
    }

    @Test
    @DisplayName('byFields succeeds when no documents match an IN_ARRAY comparison')
    void byFieldsNoMatchInArray() {
        new PgDB().withCloseable JsonFunctions.&byFieldsNoMatchInArray
    }

    @Test
    @DisplayName('byContains retrieves matching documents')
    void byContainsMatch() {
        new PgDB().withCloseable JsonFunctions.&byContainsMatch
    }

    @Test
    @DisplayName('byContains retrieves ordered matching documents')
    void byContainsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&byContainsMatchOrdered
    }

    @Test
    @DisplayName('byContains succeeds when no documents match')
    void byContainsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&byContainsNoMatch
    }

    @Test
    @DisplayName('byJsonPath retrieves matching documents')
    void byJsonPathMatch() {
        new PgDB().withCloseable JsonFunctions.&byJsonPathMatch
    }

    @Test
    @DisplayName('byJsonPath retrieves ordered matching documents')
    void byJsonPathMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&byJsonPathMatchOrdered
    }

    @Test
    @DisplayName('byJsonPath succeeds when no documents match')
    void byJsonPathNoMatch() {
        new PgDB().withCloseable JsonFunctions.&byJsonPathNoMatch
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document')
    void firstByFieldsMatchOne() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsMatchOne
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many')
    void firstByFieldsMatchMany() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsMatchMany
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many (ordered)')
    void firstByFieldsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('firstByFields returns null when no document matches')
    void firstByFieldsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&firstByFieldsNoMatch
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document')
    void firstByContainsMatchOne() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsMatchOne
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document among many')
    void firstByContainsMatchMany() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsMatchMany
    }

    @Test
    @DisplayName('firstByContains retrieves a matching document among many (ordered)')
    void firstByContainsMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsMatchOrdered
    }

    @Test
    @DisplayName('firstByContains returns null when no document matches')
    void firstByContainsNoMatch() {
        new PgDB().withCloseable JsonFunctions.&firstByContainsNoMatch
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document')
    void firstByJsonPathMatchOne() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathMatchOne
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document among many')
    void firstByJsonPathMatchMany() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathMatchMany
    }

    @Test
    @DisplayName('firstByJsonPath retrieves a matching document among many (ordered)')
    void firstByJsonPathMatchOrdered() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathMatchOrdered
    }

    @Test
    @DisplayName('firstByJsonPath returns null when no document matches')
    void firstByJsonPathNoMatch() {
        new PgDB().withCloseable JsonFunctions.&firstByJsonPathNoMatch
    }
}
