package solutions.bitbadger.documents.groovy.tests.integration

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import solutions.bitbadger.documents.DocumentException

import static org.junit.jupiter.api.Assertions.assertThrows

/**
 * SQLite integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName('Groovy | SQLite: Json')
final class SQLiteJsonIT {

    @Test
    @DisplayName('all retrieves all documents')
    void allDefault() {
        new SQLiteDB().withCloseable JsonFunctions.&allDefault
    }

    @Test
    @DisplayName('all succeeds with an empty table')
    void allEmpty() {
        new SQLiteDB().withCloseable JsonFunctions.&allEmpty
    }

    @Test
    @DisplayName('byId retrieves a document via a string ID')
    void byIdString() {
        new SQLiteDB().withCloseable JsonFunctions.&byIdString
    }

    @Test
    @DisplayName('byId retrieves a document via a numeric ID')
    void byIdNumber() {
        new SQLiteDB().withCloseable JsonFunctions.&byIdNumber
    }

    @Test
    @DisplayName('byId returns null when a matching ID is not found')
    void byIdNotFound() {
        new SQLiteDB().withCloseable JsonFunctions.&byIdNotFound
    }

    @Test
    @DisplayName('byFields retrieves matching documents')
    void byFieldsMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatch
    }

    @Test
    @DisplayName('byFields retrieves ordered matching documents')
    void byFieldsMatchOrdered() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatchOrdered
    }

    @Test
    @DisplayName('byFields retrieves matching documents with a numeric IN clause')
    void byFieldsMatchNumIn() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatchNumIn
    }

    @Test
    @DisplayName('byFields succeeds when no documents match')
    void byFieldsNoMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsNoMatch
    }

    @Test
    @DisplayName('byFields retrieves matching documents with an IN_ARRAY comparison')
    void byFieldsMatchInArray() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsMatchInArray
    }

    @Test
    @DisplayName('byFields succeeds when no documents match an IN_ARRAY comparison')
    void byFieldsNoMatchInArray() {
        new SQLiteDB().withCloseable JsonFunctions.&byFieldsNoMatchInArray
    }

    @Test
    @DisplayName('byContains fails')
    void byContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.byContainsMatch db }
        }
    }

    @Test
    @DisplayName('byJsonPath fails')
    void byJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.byJsonPathMatch db }
        }
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document')
    void firstByFieldsMatchOne() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsMatchOne
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many')
    void firstByFieldsMatchMany() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsMatchMany
    }

    @Test
    @DisplayName('firstByFields retrieves a matching document among many (ordered)')
    void firstByFieldsMatchOrdered() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsMatchOrdered
    }

    @Test
    @DisplayName('firstByFields returns null when no document matches')
    void firstByFieldsNoMatch() {
        new SQLiteDB().withCloseable JsonFunctions.&firstByFieldsNoMatch
    }

    @Test
    @DisplayName('firstByContains fails')
    void firstByContainsFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.firstByContainsMatchOne db }
        }
    }

    @Test
    @DisplayName('firstByJsonPath fails')
    void firstByJsonPathFails() {
        new SQLiteDB().withCloseable { db ->
            assertThrows(DocumentException) { JsonFunctions.firstByJsonPathMatchOne db }
        }
    }
}
