package solutions.bitbadger.documents.kotlinx.tests.integration

import org.junit.jupiter.api.DisplayName
import kotlin.test.Test

/**
 * PostgreSQL integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("KotlinX | PostgreSQL: Json")
class PostgreSQLJsonIT {

    @Test
    @DisplayName("all retrieves all documents")
    fun allDefault() =
        PgDB().use(JsonFunctions::allDefault)

    @Test
    @DisplayName("all succeeds with an empty table")
    fun allEmpty() =
        PgDB().use(JsonFunctions::allEmpty)

    @Test
    @DisplayName("byId retrieves a document via a string ID")
    fun byIdString() =
        PgDB().use(JsonFunctions::byIdString)

    @Test
    @DisplayName("byId retrieves a document via a numeric ID")
    fun byIdNumber() =
        PgDB().use(JsonFunctions::byIdNumber)

    @Test
    @DisplayName("byId returns null when a matching ID is not found")
    fun byIdNotFound() =
        PgDB().use(JsonFunctions::byIdNotFound)

    @Test
    @DisplayName("byFields retrieves matching documents")
    fun byFieldsMatch() =
        PgDB().use(JsonFunctions::byFieldsMatch)

    @Test
    @DisplayName("byFields retrieves ordered matching documents")
    fun byFieldsMatchOrdered() =
        PgDB().use(JsonFunctions::byFieldsMatchOrdered)

    @Test
    @DisplayName("byFields retrieves matching documents with a numeric IN clause")
    fun byFieldsMatchNumIn() =
        PgDB().use(JsonFunctions::byFieldsMatchNumIn)

    @Test
    @DisplayName("byFields succeeds when no documents match")
    fun byFieldsNoMatch() =
        PgDB().use(JsonFunctions::byFieldsNoMatch)

    @Test
    @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
    fun byFieldsMatchInArray() =
        PgDB().use(JsonFunctions::byFieldsMatchInArray)

    @Test
    @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
    fun byFieldsNoMatchInArray() =
        PgDB().use(JsonFunctions::byFieldsNoMatchInArray)

    @Test
    @DisplayName("byContains retrieves matching documents")
    fun byContainsMatch() =
        PgDB().use(JsonFunctions::byContainsMatch)

    @Test
    @DisplayName("byContains retrieves ordered matching documents")
    fun byContainsMatchOrdered() =
        PgDB().use(JsonFunctions::byContainsMatchOrdered)

    @Test
    @DisplayName("byContains succeeds when no documents match")
    fun byContainsNoMatch() =
        PgDB().use(JsonFunctions::byContainsNoMatch)

    @Test
    @DisplayName("byJsonPath retrieves matching documents")
    fun byJsonPathMatch() =
        PgDB().use(JsonFunctions::byJsonPathMatch)

    @Test
    @DisplayName("byJsonPath retrieves ordered matching documents")
    fun byJsonPathMatchOrdered() =
        PgDB().use(JsonFunctions::byJsonPathMatchOrdered)

    @Test
    @DisplayName("byJsonPath succeeds when no documents match")
    fun byJsonPathNoMatch() =
        PgDB().use(JsonFunctions::byJsonPathNoMatch)

    @Test
    @DisplayName("firstByFields retrieves a matching document")
    fun firstByFieldsMatchOne() =
        PgDB().use(JsonFunctions::firstByFieldsMatchOne)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many")
    fun firstByFieldsMatchMany() =
        PgDB().use(JsonFunctions::firstByFieldsMatchMany)

    @Test
    @DisplayName("firstByFields retrieves a matching document among many (ordered)")
    fun firstByFieldsMatchOrdered() =
        PgDB().use(JsonFunctions::firstByFieldsMatchOrdered)

    @Test
    @DisplayName("firstByFields returns null when no document matches")
    fun firstByFieldsNoMatch() =
        PgDB().use(JsonFunctions::firstByFieldsNoMatch)

    @Test
    @DisplayName("firstByContains retrieves a matching document")
    fun firstByContainsMatchOne() =
        PgDB().use(JsonFunctions::firstByContainsMatchOne)

    @Test
    @DisplayName("firstByContains retrieves a matching document among many")
    fun firstByContainsMatchMany() =
        PgDB().use(JsonFunctions::firstByContainsMatchMany)

    @Test
    @DisplayName("firstByContains retrieves a matching document among many (ordered)")
    fun firstByContainsMatchOrdered() =
        PgDB().use(JsonFunctions::firstByContainsMatchOrdered)

    @Test
    @DisplayName("firstByContains returns null when no document matches")
    fun firstByContainsNoMatch() =
        PgDB().use(JsonFunctions::firstByContainsNoMatch)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document")
    fun firstByJsonPathMatchOne() =
        PgDB().use(JsonFunctions::firstByJsonPathMatchOne)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many")
    fun firstByJsonPathMatchMany() =
        PgDB().use(JsonFunctions::firstByJsonPathMatchMany)

    @Test
    @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
    fun firstByJsonPathMatchOrdered() =
        PgDB().use(JsonFunctions::firstByJsonPathMatchOrdered)

    @Test
    @DisplayName("firstByJsonPath returns null when no document matches")
    fun firstByJsonPathNoMatch() =
        PgDB().use(JsonFunctions::firstByJsonPathNoMatch)
}