package solutions.bitbadger.documents.scala.tests.integration

import org.junit.jupiter.api.{DisplayName, Test}

import scala.util.Using

/**
 * PostgreSQL integration tests for the `Json` object / `json*` connection extension functions
 */
@DisplayName("Scala | PostgreSQL: Json")
class PostgreSQLJsonIT:

  @Test
  @DisplayName("all retrieves all documents")
  def allDefault(): Unit =
    Using(PgDB()) { db => JsonFunctions.allDefault(db) }

  @Test
  @DisplayName("all succeeds with an empty table")
  def allEmpty(): Unit =
    Using(PgDB()) { db => JsonFunctions.allEmpty(db) }

  @Test
  @DisplayName("byId retrieves a document via a string ID")
  def byIdString(): Unit =
    Using(PgDB()) { db => JsonFunctions.byIdString(db) }

  @Test
  @DisplayName("byId retrieves a document via a numeric ID")
  def byIdNumber(): Unit =
    Using(PgDB()) { db => JsonFunctions.byIdNumber(db) }

  @Test
  @DisplayName("byId returns null when a matching ID is not found")
  def byIdNotFound(): Unit =
    Using(PgDB()) { db => JsonFunctions.byIdNotFound(db) }

  @Test
  @DisplayName("byFields retrieves matching documents")
  def byFieldsMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.byFieldsMatch(db) }

  @Test
  @DisplayName("byFields retrieves ordered matching documents")
  def byFieldsMatchOrdered(): Unit =
    Using(PgDB()) { db => JsonFunctions.byFieldsMatchOrdered(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with a numeric IN clause")
  def byFieldsMatchNumIn(): Unit =
    Using(PgDB()) { db => JsonFunctions.byFieldsMatchNumIn(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match")
  def byFieldsNoMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.byFieldsNoMatch(db) }

  @Test
  @DisplayName("byFields retrieves matching documents with an IN_ARRAY comparison")
  def byFieldsMatchInArray(): Unit =
    Using(PgDB()) { db => JsonFunctions.byFieldsMatchInArray(db) }

  @Test
  @DisplayName("byFields succeeds when no documents match an IN_ARRAY comparison")
  def byFieldsNoMatchInArray(): Unit =
    Using(PgDB()) { db => JsonFunctions.byFieldsNoMatchInArray(db) }

  @Test
  @DisplayName("byContains retrieves matching documents")
  def byContainsMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.byContainsMatch(db) }

  @Test
  @DisplayName("byContains retrieves ordered matching documents")
  def byContainsMatchOrdered(): Unit =
    Using(PgDB()) { db => JsonFunctions.byContainsMatchOrdered(db) }

  @Test
  @DisplayName("byContains succeeds when no documents match")
  def byContainsNoMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.byContainsNoMatch(db) }

  @Test
  @DisplayName("byJsonPath retrieves matching documents")
  def byJsonPathMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.byJsonPathMatch(db) }

  @Test
  @DisplayName("byJsonPath retrieves ordered matching documents")
  def byJsonPathMatchOrdered(): Unit =
    Using(PgDB()) { db => JsonFunctions.byJsonPathMatchOrdered(db) }

  @Test
  @DisplayName("byJsonPath succeeds when no documents match")
  def byJsonPathNoMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.byJsonPathNoMatch(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document")
  def firstByFieldsMatchOne(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByFieldsMatchOne(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many")
  def firstByFieldsMatchMany(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByFieldsMatchMany(db) }

  @Test
  @DisplayName("firstByFields retrieves a matching document among many (ordered)")
  def firstByFieldsMatchOrdered(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByFieldsMatchOrdered(db) }

  @Test
  @DisplayName("firstByFields returns null when no document matches")
  def firstByFieldsNoMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByFieldsNoMatch(db) }

  @Test
  @DisplayName("firstByContains retrieves a matching document")
  def firstByContainsMatchOne(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByContainsMatchOne(db) }

  @Test
  @DisplayName("firstByContains retrieves a matching document among many")
  def firstByContainsMatchMany(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByContainsMatchMany(db) }

  @Test
  @DisplayName("firstByContains retrieves a matching document among many (ordered)")
  def firstByContainsMatchOrdered(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByContainsMatchOrdered(db) }

  @Test
  @DisplayName("firstByContains returns null when no document matches")
  def firstByContainsNoMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByContainsNoMatch(db) }

  @Test
  @DisplayName("firstByJsonPath retrieves a matching document")
  def firstByJsonPathMatchOne(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByJsonPathMatchOne(db) }

  @Test
  @DisplayName("firstByJsonPath retrieves a matching document among many")
  def firstByJsonPathMatchMany(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByJsonPathMatchMany(db) }

  @Test
  @DisplayName("firstByJsonPath retrieves a matching document among many (ordered)")
  def firstByJsonPathMatchOrdered(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByJsonPathMatchOrdered(db) }

  @Test
  @DisplayName("firstByJsonPath returns null when no document matches")
  def firstByJsonPathNoMatch(): Unit =
    Using(PgDB()) { db => JsonFunctions.firstByJsonPathNoMatch(db) }
